;****************************************
;Flash include module for loader for DS87C520/DS87C530/DS83C520
;Version 0.0
;Revision history
;Date      Resp  Revision
;06/27/95  KS    Creation
;****************************************
; General notes:
; This module contains the subroutines to allow the High-Speed Microcontroller
; to be used as a bootloader for an AMD29F010 Flash memory.
;***************************
;Embedded Byte Program Algorithm. On entry - ACC - byte to be written
;                                            DPTR0 - Address to be written

w_byte:
    push   acc           ;Save data because we need accumulator
    mov    dps, #01h     ;Switch to DPTR1

    mov    dptr, #05555h ;First Bus Write cycle = write AAh to 5555h
    mov    a, #0AAh
    movx   @dptr, a

    mov    dptr, #02AAAh ;Second Bus Write cycle = write 55h to 2AAAh
    mov    a,#055h
    movx   @dptr, a

    mov    dptr, #05555h ;Third Bus Write cycle = write A0h to 5555h
    mov    a, #0A0h
    movx   @dptr, a

    inc    dps           ;Now write the byte. Switch back to DPTR0 and get
    pop    acc           ; the data byte off the stack and program it.
    movx   @dptr, a

    call   prog_verify   ;verify operation complete spleeg add error handler.
    ret

;***************************
;Embedded Chip Erase algorthim.

chip_erase:
    mov    dptr, #05555h ;First Bus Write cycle = write AAh to 5555h.
    mov    a, #0AAh
    movx   @dptr, a

    mov    dptr, #02AAAh ;Second Bus Write cycle = write 55h to 2AAAh.
    mov    a, #055h
    movx   @dptr, a

    mov    dptr, #05555h ;Third Bus Write cycle = write 80h to 5555h.
    mov    a, #080h
    movx   @dptr, a

    mov    a, #0AAh      ;Fourth Bus Write cycle = write AAh to 5555h.
    movx   @dptr, a

    mov    dptr, #02AAAh ;Fifth Bus Write cycle = write 55h to 2AAAh.
    cpl    a
    movx   @dptr, a

    mov    dptr, #05555h ;Sixth Bus Write cycle = write 10h to 5555h.
    mov    a, #010h
    movx   @dptr, a

    mov    a, #80h       ;Set DQ7 = 1 for data\ polling method.
    call   prog_verify   ;Verify the programming operation.
    ret

;***************
;Program verify - This uses the data\ polling method to verify that the
;                 programming was successful.
prog_verify:
    jb     acc.7, DQ7_hi_test   ;To speed up code, use two separate routines
                                ; for high and low DQ7.
DQ7_lo_test:
    movx   a, @dptr             ;Poll D7 until it goes low.
    jnb    acc.7, prog_complete
    jnb    acc.5, DQ7_lo_test   ;If DQ5=0, no programming error. Keep polling.
    movx   a, @dptr             ;Possible programming error. Check data one
    jnb    acc.7,prog_complete  ; more time. If DQ7 is low, then okay.
    sjmp   prog_fail

DQ7_hi_test:
    movx   a, @dptr             ;Poll D7 until it goes high.
    jb     acc.7, prog_complete
    jnb    acc.5, DQ7_hi_test   ;If DQ5=0, no programming error. Keep polling.
    movx   a, @dptr             ;Possible programming error. Check data one
    jb     acc.7, prog_complete ; more time. If DQ7 is high, then okay.

prog_fail:
    setb   errflg               ;Flag error for main routine
prog_complete:
    movx   a, @dptr             ;Read data back to restore data in acc.
    ret

;***************************
;Fill byte command is not supported on Flash devices.
f_byte:
    ljmp   cmd_bad

end
