;****************************************
;Bootstrap loader for DS87C520/DS87C530/DS83C520
;Version 0.0
;Revision history
;Date      Resp  Revision
;01/01/95  CS    Creation
;06/27/95  KS    Revised to support AMD29F010 module
;****************************************
; General notes:
; This program communicates through serial port1 to minimize the impact on
; existing 8051 designs. The constant baud_rate can be changed to provide
; a variety of baud rates with different crystals. Be aware that the flash
; programming algorithm is very slow, and faster baud rates may not provide
; much improvement.
;****************************************
;      User-definable constants
;------------------------------------------------------------------
baud_rate       equ     0FAh            ; 9600 with SMOD_1=1 at 11.059 MHz
test_bit        equ     97h             ;Use P1.7 to invoke loader
restart_addr    equ     0FFF0h          ;Location of external restart code
;------------------------------------------------------------------
;       ascii equates 
;------------------------------------------------------------------
ctrlc   equ    003h                     ; control c
bs      equ    008h                     ; back space 
tab     equ    009h                     ; horizontal tab, ^i 
lf      equ    00ah                     ; line feed
cr      equ    00dh                     ; carriage return
xon     equ    011h                     ; rs232 protocol ^q
xoff    equ    013h                     ; rs232 protocol ^s
spa     equ    020h                     ; space
colon   equ    03ah                     ; ':'
del     equ    07fh                     ; delete character 
  
;------------------------------------------------------------------
;       register usage for bank 0 and bank 1
;------------------------------------------------------------------
; getadr   r0  -> address to fetch char from 
; putadr   r1  -> address to put char to 
; nchs     r2  -> number of chars in buffer
; bufbase  r3  -> base address of buffer 
; tempch   r3  -> temp resting place for cmdlin ch 
; seen     r3  -> number of chars seen when decoding number
; getoff   r4  -> offset into buffer to fetch from 
; putoff   r5  -> buffer to put to 
; ch       r6  -> char from input interrupt
  
;------------------------------------------------------------------
;       equivalenced to registers r4 and r5 in bank 0
;------------------------------------------------------------------
getoffd equ    004h                     ; direct byte to r4 of bank 0 
putoffd equ    005h                     ; direct byte to r5 of bank 0 
  
;------------------------------------------------------------------
;       command line buffer data structures
;------------------------------------------------------------------
clbuf   equ    00bh                     ; byte before command line buffer 
clsiz   equ    012h                     ; size of command line buffer 
endclb  equ    clbuf+clsiz              ; last byte in command line buffer
  
;------------------------------------------------------------------
;       bit flags 
;------------------------------------------------------------------
comflg  equ    021h                     ; bits 8 through 0fh
xmt     equ    008h                     ; set when xmitting, clear otherwise
xmoff   equ    009h                     ; set when xoff is received--don't xmit 
  
edtflg  equ    comflg+1                 ; bits 010h through 017h
lodver  equ    010h                     ;   load/verify flag 0/1
eol     equ    011h                     ;   set when at command line terminator 
errflg  equ    012h                     ;   set when error conditon if found
hexch   equ    013h                     ;   set when valid hex char seen
  
trace   equ    018h                     ;   set when echoing incoming hex data
  
;------------------------------------------------------------------
;       2 byte variables
;------------------------------------------------------------------
here    equ    0024h                    ; start after flag bits
  
lo      equ    here+000h                ; used to decode 16 bit numbers
hi      equ    here+001h                ;   overlayed on diff
crc_lo  equ    here+000h                ; lo byte of crc calculation
crc_hi  equ    here+001h                ;   hi part of crc calculation
diff_l  equ    here+000h                ; calculated word difference 
diff_h  equ    here+001h                ;
start_l equ    here+002h                ; address to start from
start_h equ    here+003h                ;
stop_l  equ    here+004h                ; address to stop at 
stop_h  equ    here+005h                ;
cmp_l   equ    here+006h                ; word to compare stop to
cmp_h   equ    here+007h                ;
  
;------------------------------------------------------------------
;       1 byte variables
;------------------------------------------------------------------
cksum   equ    here+00ah                ; sum of bytes in stohex and lodhex
curcksum equ   here+00bh                ; current calculated check sum on line 
len     equ    here+00ch                ; number of bytes on line
rectyp  equ    here+00dh                ; intel hex record type
val     equ    here+00eh                ; temp used: lodhex
tmp     equ    here+00fh                ; temp used: getbyt, gclwrd, putbyt
  
buf     equ    040h                     ; input character buffer base
bsiz    equ    020h                     ; buffer size of 32 bytes
  
stkadr  equ    060h                     ; move stack base to here
  
;------------------------------------------------------------------
;    equates for register/bit names unique to the High-Speed Micros
;------------------------------------------------------------------
ta	 equ	 0c7h			 ;Timed Access register
dps      equ     86h                     ;Data Pointer Select reg
scon1    equ     0C0h                    ;Serial Port 1 control reg
sbuf1    equ     0C1h                    ;Serial Port 1 data buffer
ri1      equ     0C0h                    ;Ser Port 1 receiver int flag
ti1      equ     0C1h                    ;Ser Port 1 xmitter int flag
rb8_1	 equ	 0C2h			 ;Ser Port 1 rcvr bit 8 flag
es1	 equ	 0AEh			 ;Ser Port 1 int enable bit
smod_1	 equ	 0DFh			 ;Ser Port 1 baud rate doubler bit
rwt      equ     0D8h                    ;Restart Watchdog timer bit.
ewt      equ     0D9h                    ;Watchdog timer reset enable bit.
wdcon    equ     0D8h                    ;Watchdog control register
rcap2l   equ     0CAh                    ;Timer 2 capture. Used as temp reg.
;------------------------------------------------------------------
; Reset Vector
;------------------------------------------------------------------
 cseg	at 0			   ; reset vector
 ajmp	 start

;------------------------------------------------------------------
;   name: serial_int
; 
;   abstract: serial int 1 processing. If we are interrupted
;             on a transmit, clear the flag and return. If we get
;             a character, grab it up and put it into the receive 
;             buffer. If the buffer is full, throw the data away.
;             register bank 0 is used for the receive buffer vars.
;             do not echo here. The following characters are read:
;                  ^c: push init; set bit reset; reti 
;                 xon: clear bit xmoff; reti
;                xoff: set bit xmoff; reti
;                  ^i: tabs are converted to spaces 
;                 del: converted to backspaces
;                 a-z: lower case alpha converted to upper
;                 bs,cr,spa,:,0-9,A-Z: inserted into buffer 
;             everything else is thrown away. 
;
;   variables modified: register bank 0 
;------------------------------------------------------------------
        cseg at 03Bh                    ;Serial Port 1 interrupt routine
serial_int:
        clr   es1                       ;Disable more serial interrupts
        push  psw                       ;Save program status register
        push  acc                       ;Save acc, we are going to change it

        jnb   ti1,ser_a                 ;If ti=0 goto ser_a
        clr   ti1                       ;We must have finished a transmit
        clr   xmt                       ; clear our bit
        ajmp  ser_x                     ; and return
ser_a:
        jnb   ri1,ser_xx                ;If ri=0 goto ser_x, indirectly
        clr   ri1                       ; clear the receive bit
  
        mov   a,sbuf1                   ;Get the character
        jb    rb8_1, ser_ab             ;If valid stop bit then continue
        cjne  a,#0,ser_ab               ; if not NULL then continue
        mov   ie,a                      ; BREAK char, now disable interrupts

        jnb   p1.2, $                   ;Wait until RI goes high again
        reti
ser_ab:
        clr   acc.7                     ;Make this 0-127
  
        cjne  a,#xon,ser_b              ;If a <> xon then goto ser_b
        clr   xmoff                     ;   clear the bit
ser_xx:                                 ;   (couldn't get there in one branch)
        ajmp  ser_x                     ;   and exit this interrupt
ser_b:
        cjne  a,#xoff,ser_c             ;If a <> xoff then goto ser_c
        setb  xmoff                     ;   let putch know not to xmit
        ajmp  ser_x                     ;   any more chars and exit
ser_c:
        cjne  a,#ctrlc,ser_d            ;If a <> ctrlc then goto ser_d
        jnb   xmt,ser_c0                ;   if a char is not being xmted, jmp
        jnb   ti1, $                    ;    wait till xmit is done to cntrl c
ser_c0:
        mov   dptr,#sl_init             ;   get init address
        push  dpl                       ;   push low byte of address
        push  dph                       ;   and the high byte
        reti                            ;   then return thru init
ser_d:
        cjne  a,#del,ser_e              ;If a <> del then goto ser_e
        mov   a,#bs                     ;   else a = bs
        ajmp  ser_o                     ;        goto insert [bs]
ser_e:
        cjne  a,#'a',ser_f              ;If a < 'a'
ser_f:
        jc    ser_g                     ;   then goto ser_g
        clr   acc.5                     ;   else fold to uppercase
ser_g:
        cjne  a,#('Z'+1),ser_h          ;If a >= 'Z'+1
ser_h:
        jnc   ser_x                     ;   then exit
  
        cjne  a,#'A',ser_i              ;If a >= 'A'
ser_i:
        jnc   ser_o                     ;   then insert it [A-Z]
  
        cjne  a,#(':'+1),ser_j          ;If a >= ':'+1
ser_j:
        jnc   ser_x                     ;   then exit
  
        cjne  a,#'0',ser_k              ;If a >= '0'
ser_k:
        jnc   ser_o                     ;   then insert it [0-9,:]
  
        cjne  a,#tab,ser_l              ;If a <> tab
        mov   a,#spa                    ;   then a = spa
ser_l:
        cjne  a,#spa,ser_m              ;If a <> space then goto ser_m
        ajmp  ser_o                     ;   then insert it [spa]
ser_m:
        cjne  a,#cr,ser_n               ;If a <> cr then goto ser_n
        ajmp  ser_o                     ;   then insert it [cr]
ser_n:
        cjne  a,#bs,ser_x               ;If a <> bs then exit
ser_o:
        clr   rs0                       ;Use bank 0 registers (assume rs1=0)
        cjne  r2,#bsiz,ser_p            ;Is the buffer full?
        ajmp  ser_x                     ;No space left, throw char away
ser_p:
        mov   r6,a                      ;Save the character
        mov   a,r3                      ;put base in accumulator
        add   a,r5                      ;ea = base + put-offset
        mov   r1,a                      ;Save the put address
        inc   r5                        ;point to the next put place
        anl   putoffd,#01fh             ;   mod (putoff, 32) using direct byte
        mov   a,r6                      ;restore character
        mov   @r1,a                     ;put the character into buff
        inc   r2                        ;++n; characters is one more
ser_x:
        pop   acc                       ;get the accumulator back
        pop   psw                       ;restore program status
        setb  es1                       ;Start up serial interrupts again
        reti                            ;return from interrupt
  
;------------------------------------------------------------------
;   name: sl_init 
; 
;   abstract: This is the entry point into the serial loader.
;             direct memory is cleared, stack and buffer is 
;             reset. Serial interrupt 1 is enabled.
;   assumptions: !!!
;           - timer 1 is set to mode 2 (8 bit reload),
;           - timers are turned on, 
;           - the serial port is enabled and in mode 1 (variable baud rate),
;           - the smod bit is set, and
;           - the reload value is in th1
;------------------------------------------------------------------
start:
        jnb     test_bit, init_loader   ;Do quick test to see if loader
	ljmp	restart_addr		; should be invoked.
init_loader:
        mov     rcap2l,wdcon            ;Save state of wdcon in temp register
                                        ; so it can be restored on exit.
        mov     ta, #0AAh               ;Disable watchdog timer reset in case
        mov     ta, #55h                ; it is enabled by default.
        clr     ewt

        setb    smod_1                  ;Turn on the SMOD bit for serial port 1
        mov     scon1, #050h            ; serial mode 1, 8 data, 1 start,stop
        mov     tmod, #021h             ; timers 0-16 bit; 1-8 bit reload
        mov	th1,  #baud_rate	; user supplied
        mov     tcon, #050h             ; timer on

        jnb     ri1, $                  ;Wait for any character before starting.
        clr     ri1                     ;Clear receive flag & throw away
        mov     a, sbuf1                ; trash character.
sl_init:
        mov   ie,#0                     ;Disable all interrupts.
        clr   a                         ;Clear all of direct memory.
        mov   psw,a                     ;Use register bank 0.
        mov   r0,#07fh                  ;Point to end of direct memory (127.)
sl_0:
        mov   @r0,a                     ;Clear direct memory
        djnz  r0,sl_0                   ; to bottom of memory.
                                        ;Set up data structures.
        mov   r3,#buf                   ;Buffer starts here.
        mov   sp,#stkadr                ;Reset stack.
  
        anl   scon1,#0fch               ;Clear ri1, ti1 just to be safe.
  
        mov   ip,a                      ;Clear the interrupt priority table.
        mov   ie,#0C0h                  ;Enable serial port 1 interrupt.
;------------------------------------------------------------------
;   name: 
; 
;   abstract: Command line processing takes place here. A prompt is displayed
;             A command line is assembled and echoed, then processed.
; 
;   inputs: none
;   outputs:
;   variables modified: 
;   error exits: none 
;   subroutines accessed directly: putstr, putch, getch, docmd
;------------------------------------------------------------------
        mov   dptr,#banner              ;Print the banner.
        acall putstr                    ;
  
        setb  rs0                       ;Use register bank 1.
cl_c:
        mov   dptr,#prompt              ;Print the prompt.
        acall putstr                    ;
        clr   a                         ;Init the command line.
        mov   r2,a                      ;No chars yet in command line.
        mov   edtflg,a                  ;Clear all of the edit flags.
                                        ; fill up the command line buffer
cl_d:
        acall getch                     ;    getch 
        cjne  a,#bs,cl_e                ;    if ch <> bs then goto cl_e
        mov   a,r2                      ;    a := nchs 
        jz    cl_d                      ;    if nchs = 0 then goto cl_d
                                        ;    /* process backspace char */
        dec   r2                        ;    --nchs
        mov   dptr,#backup              ;    do a destructive
        acall putstr                    ;    backspace 
        ajmp  cl_d                      ;    goto cl_d 
cl_e:
        mov   r3,a                      ;    save a
        inc   r2                        ;    ++nchs
        mov   a,#clbuf                  ;    a := clbase 
        add   a,r2                      ;    a := clbase + nchs
        mov   r1,a                      ;    buffer address to putadr
        mov   a,r3                      ;    get input char back 
        mov   @r1,a                     ;    save it to the command line buffer
        acall putch                     ;    echo it (cr's are made lf's)
        cjne  a,#lf,cl_f                ; until (ch = lf) or ... 
                                        ; (putch converts cr's to lf's)
        acall docmd                     ;Now execute the command line.
        jb    errflg,cl_i               ;If error, display it,
        ajmp  cl_c                      ; if not get another command line.
cl_f:
        cjne  r2,#clsiz,cl_d            ; ... or (nchs <> clsiz) then getch
cl_h:
        acall getch                     ;Start getting characters
        acall putch                     ; and echoing them,
        cjne  a,#lf,cl_h                ; until a cr is found (mod by putch).
  
        mov   dptr,#em_badcmd           ;Error: line is too long.
cl_i:
        acall putstr                    ;Print the error message.
        ajmp  cl_c                      ;Go get another command line.
  
        
;------------------------------------------------------------------
;   name: docmd 
; 
;   abstract: parse the command line and execute if possible.
;
;   inputs: command line to be processed
;   outputs: none 
;   variables modified: bank 1 registers
;   error exits: depends on command 
;   subroutines accessed directly: many.
;
;commands: l  load          e   erase       g  get port values
;          v  verify        c   calc CRC    t  toggle trace flag
;          x  Exit Loader   f   fill        p  put port values
;          d  dump
;------------------------------------------------------------------
docmd:
        mov   endclb,#lf                ;Put lf at end of buffer.
        mov   r0, #(clbuf+1)            ; get address of command line buffer 
        acall skipspa                   ; skip the white space 
        jnb   eol,do_a                  ;If at eol then
        ret                             ;    return home for another cl
do_a:
        mov   dptr,#em_extarg           ; assume error 
        setb  errflg                    ;
        acall gclch                     ; get a command line character 
        acall skipspa                   ; skip the white space 
        push  acc                       ; save the cmd 
do_b:
        pop   acc                       ; continue 
  
;-------------------------------------------------------
; l - load intel hex data to data memory
;-------------------------------------------------------
cmd_l:
        cjne  a,#'L',cmd_v              ; if cmd <> L goto cmd_v 
        acall chkeol                    ; check for end of line
        acall lodhex                    ; load the hex data by default 
        ret                             ; return 
  
;-------------------------------------------------------
; v - verify memory against intel hex data
;-------------------------------------------------------
cmd_v:
        cjne  a,#'V',cmd_g              ; if cmd <> T goto cmd_g 
        acall chkeol                    ; check for end of line
        setb  lodver                    ; set the load verify flag 
        acall lodhex                    ; load the hex data
        ret                             ; return 
  
;-------------------------------------------------------
; g - get port values and print them out ; rl2.2
;-------------------------------------------------------
cmd_g:
        cjne  a,#'G',cmd_t              ; if cmd <> G goto cmd_t 
        acall chkeol                    ; check for end of line
        mov   a,p0
        acall putbyt

        mov   a,#' '
        acall putch

        mov   a,p1
        acall putbyt

        mov   a,#' '
        acall putch

        mov   a,p2
        acall putbyt

        mov   a,#' '
        acall putch

        mov   a,p3
        acall putbyt
        ret                             ; return 
  
;-------------------------------------------------------
; t - toggle the trace flag for echoing intel hex data
;-------------------------------------------------------
cmd_t:
        cjne  a,#'T',cmd_f              ; if cmd <> T goto cmd_f
        acall chkeol                    ; check for end of line
        cpl   trace                     ; toggle the trace bit 
        mov   dptr,#onmsg               ; get on message by default
        jb    trace,cmd_t0              ; if carry is on branch
        mov   dptr,#offmsg              ; address of off message 
cmd_t0:
        acall putstr                    ; print out on off 
        ret                             ; return 
  
;-------------------------------------------------------
; f byte [start-addr [end-addr]] - fill data memory with a byte 
;  This command is not supported on flash devices.
;-------------------------------------------------------
cmd_f:
        cjne  a,#'F',cmd_p              ; if cmd <> F goto cmd_p 
        ljmp  f_byte                    ; get action from device specific
                                        ; module
;-------------------------------------------------------
; p byte byte byte byte - put bytes into p0, p1, p2 and p3 ; rl2.2
;-------------------------------------------------------
cmd_p:
        cjne  a,#'P',cmd_d              ; if cmd <> P goto cmd_d 

        acall gclwrd                    ; get the mandantory argument
        jb    errflg,cmd_px             ; if notnum then goto cmd_px 
        mov   p0,lo                     ; save fill byte in lo 

        acall skipspa                   ; move past the white space
        acall gclwrd                    ; get the mandantory argument
        jb    errflg,cmd_px             ; if notnum then goto cmd_px 
        mov   p1,lo                     ; save fill byte in lo 

        acall skipspa                   ; move past the white space
        acall gclwrd                    ; get the mandantory argument
        jb    errflg,cmd_px             ; if notnum then goto cmd_px 
	orl   lo,#0c0h			; make sure p2.7 and p2.6 are set;rl2.3
        mov   p2,lo                     ; save fill byte in lo 

        acall skipspa                   ; move past the white space
        acall gclwrd                    ; get the mandantory argument
        jb    errflg,cmd_px             ; if notnum then goto cmd_px 
	orl   lo,#0ch			; make sure p1.2 and p1.3 are set
	mov   p1,lo			; save fill byte in lo
cmd_px:
	clr   errflg			; no errors on this command
        ret                             ;
  
;-------------------------------------------------------
; d [start-addr [end-addr]] - dump memory in intel hex format 
;-------------------------------------------------------
cmd_d:
        cjne  a,#'D',cmd_c              ; if cmd <> D goto cmd_c
        acall optional                  ; see if we got some arguments 
        acall dmphex                    ; dump memory
        ret                             ;
  
;-------------------------------------------------------
; c [start-addr [end-addr]] - calculate the CRC
;   use the relations: 1 xor A = not A
;                      0 xor A = A
;-------------------------------------------------------
cmd_c:
        cjne  a,#'C',cmd_e              ; if cmd <> C goto erase
        acall optional                  ; get optional arguments if any
        clr   a                         ; clear the accumulator
        mov   crc_lo, a                 ; clear the CRC value
        mov   crc_hi, a                 ; clear the CRC value
cmd_ca:
        movx  a,@dptr                   ; move fill byte to memory 

        mov   b, #08h                   ; number of bits to crc.
crc_get_bit:
        rrc   a                         ; get low order bit into carry
        push  acc                       ; save a for later use

        jc    crc_in_1                  ; got a 1 input to crc
        mov   c, crc_lo.0               ;   xor with a 0 input bit is bit
        sjmp  crc_cont                  ;   continue
crc_in_1:
        mov   c, crc_lo.0               ;   xor with a 1 input bit
        cpl   c                         ;   is not bit.
crc_cont:
        jnc   crc_shift                 ; if carry set, just shift
        cpl   crc_hi.6                  ;   complement bit 15 of crc
        cpl   crc_lo.1                  ;   complement bit 2 of crc
crc_shift:
        mov   a, crc_hi                 ; carry is in appropriate setting
        rrc   a                         ; rotate it
        mov   crc_hi, a                 ; and save it
        mov   a, crc_lo                 ; again, carry is okay
        rrc   a                         ; rotate it
        mov   crc_lo, a                 ; and save it

        pop   acc                       ; get a back
        djnz  b, crc_get_bit            ; go get the next bit

        inc   dptr                      ; get next address 
        mov   a,cmp_h                   ; get top part to compare against 
        cjne  a,dph,cmd_ca              ; if topbytes are unequal keep storing 
        mov   a,cmp_l                   ; and now the bottom part
        cjne  a,dpl,cmd_ca              ; bottom bytes are <>, keep working 

        mov   dptr, #crcmsg             ; print CRC = 
        acall putstr                    ; do it
        mov   a, crc_hi                 ; high byte of crc calculation
        acall putbyt                    ;  print it
        mov   a, crc_lo                 ; low byte of crc calculation
        acall putbyt                    ;  print it
cmd_cx:
        ret                             ;
  
;-------------------------------------------------------
; e - erase the flash ROM
;-------------------------------------------------------
cmd_e:
        cjne  a,#'E',cmd_x              ; if cmd <> E goto exit
	acall chkeol
        call  chip_erase
        jnb   errflg,erase_ok           ;Error during programming, if so,
        mov   dptr, #em_badpgm          ; print error msg.
erase_ok:
        clr   errflg                    ;Clear error flag.
        ret
;-------------------------------------------------------
; x - exit the loader and return control to external memory
;-------------------------------------------------------
cmd_x:
	cjne  a,#'X',cmd_bad            ; if cmd <> X bad command
	acall chkeol

clr_sfr:
	clr   a 			;Restore used regs to reset state
        mov   ie,a

	mov   dps, #01h 		;Clear both data pointers
	mov   dptr, #0h
	inc   dps
	mov   dptr, #0h

	mov   tmod, a
	mov   tcon, a
	mov   tl1, a
	mov   th1, a

        mov   ip,a
        mov   scon1,a
        mov   psw,a
        mov   b,a
        mov   sp,#07h

	mov   r0, #0FFh 		;Clear the 256 byte scratchpad RAM
clr_scratch:
	mov   @r0, a
	djnz  r0, clr_scratch

        mov   ta,#0AAh                  ;First reset watchdog, then restore
        mov   ta,#55h                   ; to original state
        setb  rwt
	mov   ta,#0AAh
	mov   ta,#55h
        mov   wdcon, rcap2l
        mov   rcap2l, a                 ;Finally, clear out rcap2l, which
                                        ; had been holding wdcon values.
	ljmp  restart_addr		;We're outta here. no RET needed.
;-------------------------------------------------------
; no legal command found. 
;-------------------------------------------------------
cmd_bad:
        mov   dptr,#em_badcmd           ; illegal command was entered
        ret
  
        
;------------------------------------------------------------------
;   name: chkeol
; 
;   abstract: check the eol flag.  if it is set, pop the immediate
;             return value, and return to the next, otherwise just
;             clear the flag and return 
; 
;   inputs: errflg
;   outputs: errflg 
;   variables modified: errflg
;   error exits: none 
;   subroutines accessed directly: none 
;------------------------------------------------------------------
chkeol:
        jb    eol,chk_0                 ; if eol=set then goto chk_0 
        pop   acc                       ; not at eol, stuff still on 
        pop   acc                       ; the line. return through 
        ret                             ; previous call
chk_0:
        clr   errflg                    ; no error, yet
        ret                             ; continue.
  
        
;------------------------------------------------------------------
;   name: optional
; 
;   abstract: look on the command line for two optional arguments.
;             these values will be the start and stop address of
;             memory.  by default use 0 to memory max.  dptr is set
;             with the value of start.  clear the error flag and
;             set the error flag if we still have stuff on the
;             command line.  note that gclwrd may return an error.
;             report an error if start > stop.
;             ---- important note ----
;             whenever the error flag is set, return to the callers 
;             caller.  that is, pop the stack twice before returning. 
; 
;   inputs: none
;   outputs: start, stop, cmp, dptr 
;   variables modified: 
;   error exits: return to the callers caller 
;   subroutines accessed directly: gclwrd, skipspa
;------------------------------------------------------------------
optional: 
        clr   errflg                    ; assume no errors 
        clr   a                         ; init start, stop and cmp 
        mov   start_l,a                 ;
        mov   start_h,a                 ; start = 0 
        mov   stop_l,#0ffh              ;
        mov   stop_h,#07fh              ; stop = 07fff, 32k
        mov   cmp_l,a                   ;
        mov   cmp_h,#080h               ; stop one past

        jb    eol,opt_d                 ; if at eol then use defaults
        acall gclwrd                    ; get a number, start address
        jb    errflg,opt_r              ; got an error 
        mov   start_l,lo                ;
        mov   start_h,hi                ; got a new start value
        acall skipspa                   ;
        jb    eol,opt_d                 ; only have a start value, return
        acall gclwrd                    ; get the stop address 
        jb    errflg,opt_r              ; got an error, return 
  
        mov   a,lo                      ; got the stop value 
        mov   stop_l,a                  ;
        add   a,#01h                    ; and begin calc of cmp value
        mov   cmp_l,a                   ; got the low part of cmp
        mov   stop_h,hi                 ; stop is done 
        clr   a                         ; need the carry from the previous add 
        addc  a,hi                      ; now i have the top of cmp
        mov   cmp_h,a                   ; and we are done
 
        acall skipspa                   ; any more on the line?
        jb    eol,opt_d                 ; if at end of line then return
        mov   dptr,#em_extarg           ; error - extra stuff on line
        ajmp  opt_r                     ; got to return routine
opt_d:
        mov   dpl,start_l               ; no error if this is executed.
        mov   dph,start_h               ; dptr = start 
  
        mov   a,start_h                 ; is start > stop? lets find out 
        cjne  a,stop_h,opt_f            ; if high bytes are different...
  
        mov   a,stop_l                  ; starth == stoph at this point
        cjne  a,start_l,opt_e           ; if stop < start then error
opt_e:
        jc    opt_g                     ; jump if stop_l < start_l 
        ret                             ; return, everything is fine 
opt_f:
        jnc   opt_g                     ; if start_h >= stop_h then goto opt_g 
        ret                             ; start <= stop, so return 
opt_g:
        mov   dptr,#em_illopt           ; start > stop 
opt_r:
        setb  errflg                    ; got an error 
        pop   acc                       ; get rid of the return address and
        pop   acc                       ; return to the callers caller.
opt_x:
        ret                             ;
        
;------------------------------------------------------------------
;   name: gclch - get command line character
; 
;   abstract: get the next char from the command line,
;             point to the next char.  if it is a cr then 
;             set eol.
; 
;   inputs: register bank 1 is assumed
;   outputs: a:char, r0:point to next char, eol:set if at eol 
;   variables modified: register bank 1 
;   error exits: none 
;   subroutines accessed directly: none, but fall through 
;                                  to the routine skipspa 
;------------------------------------------------------------------
gclch:
        mov   a,@r0                     ; get the char 
        inc   r0                        ; point to next char 
        ajmp  skip_a                    ; go down and test for eol 
  
        
;------------------------------------------------------------------
;   name: skipspa 
; 
;   abstract: move r0 across the spaces.  if the char we
;             land on is a cr, set the flag eol.  note that r0
;             is changed only if the current character it is pointing 
;             to is a blank.
; 
;   inputs: register bank 1 is assumed
;   outputs: r0:points to next non blank char 
;   variables modified: register bank 1 
;   error exits: none 
;   subroutines accessed directly: none 
;------------------------------------------------------------------
skipspa:
        cjne  @r0,#spa,skip_a             ; do while ch = spa
        inc   r0                          ;    ++r0
        ajmp  skipspa                     ; continue skipping white space
skip_a:
        cjne  @r0,#cr,skip_b              ; if ch = cr then
        setb  eol                         ;    set eol 
skip_b:
        ret                               ;
        
;------------------------------------------------------------------
;   name: gclwrd - get command line hexadecmial word
; 
;   abstract: read in a string of hex digits and decode them to 
;             binary.  only the last 4 digits count.  set errflg
;             and error message if we can't get number. clear it
;             otherwise.
; 
;   inputs: the command line
;   outputs: lo, hi 
;   variables modified: a, b, seen, tmp, errflg 
;   error exits: none 
;   subroutines accessed directly: gclch, ishexch 
;------------------------------------------------------------------
gclwrd:
        clr   a                         ;
        mov   lo,a                      ; n = 0
        mov   hi,a                      ;
        mov   r3,a                      ; seen = 0 
        clr   errflg                    ;
gclw_a:
        acall gclch                     ; get a character
        acall ishexch                   ; convert it if possible 
        jnb   hexch,gclw_b              ; convert until no more hex chars
        mov   tmp,a                     ; save our nibble
  
        mov   a,lo                      ; n = n*16 + hexnibble 
        mov   b,#010h                   ;
        mul   ab                        ;
        add   a,tmp                     ;
        mov   lo,a                      ;
        mov   a,hi                      ;
        anl   a,#0fh                    ;
        swap  a                         ;
        add   a,b                       ;
        mov   hi,a                      ; all done 
  
        inc   r3                        ; we have seen one more hex char 
        ajmp  gclw_a                    ;
gclw_b:
        dec   r0                        ; put the character back 
        cjne  r3,#0,gclw_d              ; we decoded at least one char 
gclw_c:
        setb  errflg                    ; make sure they see up above
        mov   dptr,#em_nothex           ; the string to prove it 
        ret
gclw_d:
        mov   a,@r0                     ; what char did we stop on?
        cjne  a,#spa,gclw_e             ; if ch <> spa then goto gcl_w 
        ret                             ; okay delimiter 
gclw_e:
        cjne  a,#cr,gclw_c              ; if ch <> cr then goto gclw_c 
        ret                             ; everything is okay 
  
;------------------------------------------------------------------
;   name: ishexch (a) 
; 
;   abstract: if a is a valid hexadecimal character (0-9,a-f) 
;             then set bit hexch, and convert it to its binary
;             value leaving it in a.  otherwise clear hexch 
;             and return. 
; 
;   inputs: a, and ascii char 
;   outputs: a
;   variables modified: 
;   error exits: none 
;   subroutines accessed directly:
;------------------------------------------------------------------
ishexch:
        clr   hexch                     ; assume that it isn't a hex char
        cjne  a,#'G',hex_a              ;
hex_a:
        jnc   hex_x                     ; if ch >= G then exit 
        cjne  a,#'0',hex_b              ;
hex_b:
        jc    hex_x                     ; if ch <  0 then exit 
        cjne  a,#':',hex_c              ; if ch <> : then goto hex_c 
        ret                             ;    then exit 
hex_c:
        setb  hexch                     ; yep, it is a hex char
        jnb   acc.6,hex_d               ; jump if 0-9
        add   a,#09h                    ;
hex_d:
        anl   a,#0fh                    ; get bottom 4 bits. 
hex_x:
        ret                             ; return with nibble in a
  
;------------------------------------------------------------------
;   name: getch 
; 
;   abstract: return a character from the input queue.  wait if 
;             nothing is available yet.  an async receive routine 
;             fills this buffer.  register bank 0 is used for the 
;             receive buffer variables. 
; 
;   inputs: none
;   outputs: a has the input character
;   variables modified: register bank 0 
;   error exits: none 
;   subroutines accessed directly: none 
;------------------------------------------------------------------
getch:
        push  psw                       ; save the psw 
        clr   rs0                       ; use bank 0 registers (assume rs1 = 0)
get_a:
        cjne  r2,#0,get_b               ; repeat 
        ajmp  get_a                     ; until (nchs > 0) 
get_b:
        mov   a,r3                      ; put base in accumulator
        add   a,r4                      ; ea = base + get-offset 
        mov   r0,a                      ; put address of ch in r5
        inc   r4                        ; point to the next char 
        anl   getoffd,#01fh             ;    mod (getoff, 32) using direct byte
        mov   a,@r0                     ; put the character into a 
        dec   r2                        ; --n; characters is one less
        pop   psw                       ; restore it 
        ret                             ; return with character in a 
        
;------------------------------------------------------------------
;   name: putch 
; 
;   abstract: puts a character out to the serial port.
; 
;   inputs: a
;   outputs: none 
;   variables modified: a is converted to a lf whenever 
;                       a = cr on entry.
;   error exits: none 
;   subroutines accessed directly: none 
;------------------------------------------------------------------
putch:
        jb     xmoff,$                  ; repeat unitl (xmoff = 0) 
        mov    sbuf1,a                   ; copy a to sbuf
        setb   xmt                      ; set the bit to show we are xmitting
        jb     xmt,$                    ; wait till it clears
  
        cjne   a,#cr,put_a              ; if a <> cr then goto put_a 
        mov    a,#lf                    ;
        ajmp   putch                    ; now put out a line feed
put_a:
        ret                             ; xmt is cleared by serial int routine 
  
;------------------------------------------------------------------
;   name: putbyt
; 
;   abstract: take the hex number in a and put out two hex digits.
;             add the byte in a to the running total in check sum.
; 
;   inputs: a 
;   outputs: none 
;   variables modified: a, cksum, tmp 
;   error exits: none 
;   subroutines accessed directly: putch
;------------------------------------------------------------------
putbyt:
        mov    tmp,a                    ; save the byte to be printed (push) 
        swap   a                        ; shift high nibble over 
        anl    a,#0fh                   ; now we really have just a nibble 
        add    a,#90h                   ; 90h-99h,9ah-9fh
        da     a                        ; 90h-99h,00h-05h
        addc   a,#40h                   ; d0h-d9h,41h-46h
        da     a                        ; 30h-39h,41h-46h which is 0-9,a-f ascii
        acall  putch                    ; and now its on its way.
  
        mov    a,tmp                    ; get the low nibble now 
        anl    a,#0fh                   ; now we really have just a nibble 
        add    a,#90h                   ; 90h-99h,9ah-9fh
        da     a                        ; 90h-99h,00h-05h
        addc   a,#40h                   ; d0h-d9h,41h-46h
        da     a                        ; 30h-39h,41h-46h which is 0-9,a-f ascii
        acall  putch                    ; and now its on its too.
  
        mov    a,tmp                    ; save our byte
        add    a,cksum                  ; add check sum to this byte 
        mov    cksum,a                  ; and save it back to cksum
        ret                             ;
        
;------------------------------------------------------------------
;   name: putstr
; 
;   abstract: take the string pointed to by dptr and prints it out. 
;             the string must be terminated by a null.
; 
;   inputs: dptr
;   outputs: none 
;   variables modified: a, dptr 
;   error exits: none 
;   subroutines accessed directly: putch
;------------------------------------------------------------------
putstr:
        clr    a                        ; make address offset zero 
        movc   a,@a+dptr                ; get next char to print 
        jz     puts_a                   ; when zero we are at end of string
        acall  putch                    ; write it out 
        inc    dptr                     ; point to next char 
        ajmp   putstr                   ; do it again
puts_a:
        ret                             ;
  
;------------------------------------------------------------------
;   name: lodhex
; 
;   abstract: read an intel hex format file and move the data to
;             memory if this is from a load command and verify
;             that the intel and memory are the same value. 
; 
;   inputs: lodver(bit) 
;   outputs: none 
;   variables modified: cksum, curcksum, len, rectyp, val 
;                       a, dptr 
;   error exits: errrec, errver, errcks 
;   subroutines accessed directly: getch, getbyt
;------------------------------------------------------------------
lodhex:
        acall  getch                    ; repeat getch 
        jnb    trace,lod_a              ; if noecho then goto lod_a
        acall  putch                    ; putch
lod_a:
        cjne   a,#colon,lodhex          ; until (ch = ':') 
  
        mov    cksum,#0                 ; cksum := 0 
  
        acall  getbyt                   ;
        mov    len,a                    ; len := getbyt
  
        acall  getbyt                   ;
        mov    dph,a                    ;
        acall  getbyt                   ;
        mov    dpl,a                    ; address := getbyt*256 + getbyt 
  
        acall  getbyt                   ;
        mov    rectyp,a                 ; rectyp := getbyt 
  
        mov    a,rectyp                 ;
        jz     lod_b                    ; if rectyp  = 0 then goto lod_b 
        cjne   a,#01h,lod_br            ; if rectyp <> 1 then goto lod_br
lod_b:
        mov    a,len                    ;
        jz     lod_e                    ; if len = 0 then goto lod_e 
lod_c:
        acall  getbyt                   ; repeat 
        mov    val,a                    ;    val := getbyt 
        jb     lodver,lod_d             ;    if (verify) then goto lod_c 
        call   w_byte                   ;    mem[address] := val
lod_d:
        movx   a,@dptr                  ;    a = mem[address]
        cjne   a,val,lod_ver            ;    if (val<>mem) then goto lod_ver 
        inc    dptr                     ;       increment memory address
        djnz   len,lod_c                ; until (len = 0)
lod_e:
        mov    dptr,#em_badcks          ; assume we will get a bad checksum
        mov    a,cksum                  ; get the real check sum which
        cpl    a                        ; requires that the sum be complemented
        inc    a                        ; and one added (two's complement) 
        mov    curcksum,a               ; and put it someplace safe
        acall  getbyt                   ; get line check sum 
        cjne   a,curcksum,loderr        ; if the calc <> line_cksum then errcks 
lod_f:
        mov    a,rectyp                 ; check if rectyp = 1
        cjne   a,#01h,lod_a             ; keep doing it until rectype = 1. 
  
        ret                             ; done with load.
lod_br:
        mov    dptr,#em_badrec          ; bad record type
        ajmp   loderr                   ; print it out 
lod_ver:
        mov    dptr,#em_memver          ; memory verify error
loderr:
        mov    a,#cr                    ;
        acall  putch                    ; put message on a new line
        acall  putstr                   ;
        ajmp   lodhex                   ; start work on a new line 
  
;------------------------------------------------------------------
;   name: getbyt
; 
;   abstract: get two characters from getch.  they should both
;             be hex characters.  convert them to binary. 
;             add the value to a running total in cksum.
;             *** VERY IMPORTANT NOTE: *************************
;             if either of the two characters is not a hex char,
;             do something strange:  pop the stack of the return
;             address and jump into the load hex error reporting
;             code.  this simplifies error checking.
;             --------------------------------------------------
; 
;   inputs: none
;   outputs: a
;   variables modified: a, tmp, cksum 
;   error exits: direct jump to lod_err 
;   subroutines accessed directly: getch, ishexch, putch
;------------------------------------------------------------------
getbyt:
        acall  getch                    ; get a byte into a
        jnb    trace,getb_a             ; if trace is on 
        acall  putch                    ;    then echo it. 
getb_a:
        acall  ishexch                  ; convert char to hex if possible
        jnb    hexch,getb_e             ; got an error, so go do it
        swap   a                        ; shift left into high nibble
        mov    tmp,a                    ; and save it in temporary space 
  
        acall  getch                    ; get a the second byte into a 
        jnb    trace,getb_b             ; if trace is on 
        acall  putch                    ;    then echo it. 
getb_b:
        acall  ishexch                  ; convert char to hex if possible
        jnb    hexch,getb_e             ; got an error, so go do it
  
        orl    a,tmp                    ; get high nibble and return 
        mov    tmp,a                    ; save our byte (push is better) 
        add    a,cksum                  ; add cksum to this byte 
        mov    cksum,a                  ; and save it back to cksum
        mov    a,tmp                    ; and recover our byte (push then pop) 
        ret                             ;
;------------------------------------------------------------------ 
; this is very unusual code.  pop the return stack and jump to
; the error reporting code in lodhex.  be sure to notice there is 
; no return, but a jump instead.
;------------------------------------------------------------------ 
getb_e:
        pop    acc                      ; clear high byte of return address
        pop    acc                      ; clear low  byte of return address
        mov    dptr,#em_nothex          ; found a non hex character
        ajmp   loderr                   ; handle error stuff here. 
  
;------------------------------------------------------------------
;   name: dmphex (start, stop, cmp) 
; 
;   abstract: write an intel hex format file. 
;   inputs: start_l, start_h, stop_l, stop_h, cmd_l, cmp_h
;   outputs: none 
;   variables modified: cksum, len, val 
;                       cmp_l, cmp_h, diff_l, diff_h
;                       a, dptr 
;   error exits: none 
;   subroutines accessed directly: putch, putbyt, putstr
;------------------------------------------------------------------
dmphex:
        clr    c                        ;
        mov    a,stop_l                 ;
        subb   a,dpl                    ;
        mov    diff_l,a                 ;
        mov    a,stop_h                 ;
        subb   a,dph                    ;
        mov    diff_h,a                 ; diff := stop - index 
  
        jnz    dmp_b                    ; diff > 255 
        mov    a,diff_l                 ;
        anl    a,#0e0h                  ;
        jnz    dmp_b                    ; diff >  31 
        mov    len,diff_l               ; len := diff  (0 <= diff < 32)
        inc    len                      ; want to move one more byte than len 
        ajmp   dmp_c                    ;
dmp_b:
        mov    len,#020h                ; len := 32
dmp_c:
        mov    cksum,#0                 ; cksum := 0 
        mov    a,#cr                    ;
        acall  putch                    ; start record on a new line 
        mov    a,#colon                 ;
        acall  putch                    ; header character 
        mov    a,len                    ;
        acall  putbyt                   ; length 
        mov    a,dph                    ;
        acall  putbyt                   ; address (hi byte)
        mov    a,dpl                    ;
        acall  putbyt                   ; address (low byte) 
        clr    a                        ;
        acall  putbyt                   ; record type
dmp_d:
        movx   a,@dptr                  ; repeat a := mem[dptr]
        acall  putbyt                   ;    write it out
        inc    dptr                     ;    ++index 
        djnz   len,dmp_d                ; until (len = 0) 
  
        mov    a,cksum                  ; do a two's complement of cksum by
        cpl    a                        ;   complementing
        inc    a                        ;   and adding one 
        acall  putbyt                   ; cksum
                                        ;
        mov    a,cmp_h                  ; look and see if we have more to write
        cjne   a,dph,dmphex             ;   still more bytes to read.
        mov    a,cmp_l                  ;
        cjne   a,dpl,dmphex             ;   yep, still have some more. 
  
        mov    a,#cr                    ;
        acall  putch                    ; start on a new line
        mov    dptr,#endrec             ; address of intel hex end record
        acall  putstr                   ; put out the string 
  
        ret                             ;
  
;------------------------------------------------------------------
;   data definitions
;------------------------------------------------------------------
crcmsg:    db   'CRC = ', 0
endrec:    db   ':00000001FF', cr, 0 
backup:    db   bs, ' ', bs, 0 
prompt:    db   cr, '> ', 0
banner:    db   cr, 'HIGH-SPEED MICROCONTROLLER LOADER VER 0.0 '
           db   cr, 'COPYRIGHT (C) 1995  DALLAS SEMICONDUCTOR', 0
offmsg:    db   'OFF', 0 
onmsg:     db   'ON', 0
em_memver: db   'E:VERIFY', 0
em_nothex: db   'E:NOTHEX', 0
em_badcmd: db   'E:BADCMD', 0
em_extarg: db   'E:EXTARG', 0
em_illopt: db   'E:ILLOPT', 0
em_badrec: db   'E:BADREC', 0
em_badcks: db   'E:BADCKS', 0
em_badpgm: db   'E:BADPGM', 0

;
;$INCLUDE (AM29F010.ASM)
$INCLUDE (NVRAM.ASM)
 end
